package com.example.bugbattle_sdk;

import android.app.Application;
import android.graphics.Bitmap;
import android.os.Handler;
import android.os.Looper;

import androidx.annotation.NonNull;
import org.json.JSONObject;

import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;


import bugbattle.io.bugbattle.APPLICATIONTYPE;
import bugbattle.io.bugbattle.BugBattle;

import bugbattle.io.bugbattle.BugBattleActivationMethod;
import bugbattle.io.bugbattle.BugBattleNotInitialisedException;
import bugbattle.io.bugbattle.CustomActionCallback;
import bugbattle.io.bugbattle.GetBitmapCallback;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.PluginRegistry;


/**
 * BugbattleSdkPlugin
 */
public class BugbattleSdkPlugin implements FlutterPlugin, MethodChannel.MethodCallHandler {
    private MethodChannel channel;
    private static Application application;
    private Handler uiThreadHandler = new Handler(Looper.getMainLooper());

    public static void registerWith(PluginRegistry.Registrar registrarFor) {

    }

    @Override
    public void onAttachedToEngine(@NonNull final FlutterPluginBinding flutterPluginBinding) {
        channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "bugbattle_sdk");
        channel.setMethodCallHandler(this);
        application = (Application) flutterPluginBinding.getApplicationContext();
        BugBattle.getInstance().setBitmapCallback(new GetBitmapCallback() {
            @Override
            public Bitmap getBitmap() {
                return flutterPluginBinding.getFlutterEngine().getRenderer().getBitmap();
            }
        });
    }
    private void initCustomAction () {
        BugBattle.getInstance().registerCustomAction(new CustomActionCallback() {
            @Override
            public void invoke(final String message) {

               uiThreadHandler.post(new Runnable() {
                   @Override
                   public void run() {

                       Map<String, String> map = new HashMap<>();
                        map.put("name", message);

                       System.out.println(map.get("name"));
                     channel.invokeMethod("customActionCalled", map);
                   }
               });
            }
        });
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        switch (call.method) {
            case "autoConfigure":
                BugBattle.autoConfigure((String) call.argument("apiKey"), application);
                initCustomAction();
                break;
            case "initWithToken":
                BugBattle.getInstance().setApplicationType(APPLICATIONTYPE.FLUTTER);
                if (call.argument("activationMethod").equals("SHAKE")) {
                    BugBattle.initWithToken((String) call.argument("apiKey"), BugBattleActivationMethod.SHAKE, application);
                } else if (call.argument("activationMethod").equals("SCREENSHOT")) {
                    BugBattle.initialise((String) call.argument("apiKey"), BugBattleActivationMethod.SCREENSHOT, application);
                } else {
                    BugBattle.initialise((String) call.argument("apiKey"), BugBattleActivationMethod.NONE, application);
                }
                initCustomAction();
                break;
            case "initializeMany":
                List<BugBattleActivationMethod> activationMethodsList = new LinkedList<>();
                List<String> activationMethodArgs = call.argument("activationMethods");

                for (int i = 0; i < activationMethodArgs.size(); i++) {
                    if (activationMethodArgs.get(i).equals("SHAKE")) {
                        activationMethodsList.add(BugBattleActivationMethod.SHAKE);
                    } else if (activationMethodArgs.get(i).equals("SCREENSHOT")) {
                        activationMethodsList.add(BugBattleActivationMethod.SCREENSHOT);
                    } else {
                        activationMethodsList.add(BugBattleActivationMethod.NONE);
                    }
                }

                BugBattle.initialise((String) call.argument("apiKey"), activationMethodsList.toArray(new BugBattleActivationMethod[activationMethodsList.size()]), application);
                break;
            case "startBugReporting":
                try {
                    BugBattle.getInstance().startBugReporting();
                } catch (BugBattleNotInitialisedException ex) {
                    System.out.println(ex);
                }
                break;
            case "enableReplays":
                BugBattle.getInstance().enableReplays(true);
                break;
            case "setCustomerEmail":
                BugBattle.getInstance().setCustomerEmail((String) call.argument("email"));
                break;
            case "attachCustomData":
                JSONObject json = new JSONObject((Map) call.argument("customData"));
                BugBattle.getInstance().appendCustomData(json);
                break;
            case "enablePrivacyPolicy":
                BugBattle.getInstance().enablePrivacyPolicy((boolean) call.argument("enable"));
                break;
            case "setPrivacyPolicyUrl":
                BugBattle.getInstance().setPrivacyPolicyUrl((String) call.argument("url"));
                break;
            case "setApiUrl":
                BugBattle.getInstance().setApiUrl((String) call.argument("url"));
                break;
            case "setLanguage":
                BugBattle.getInstance().setLanguage((String) call.argument("language"));
                break;
            case "sendSilentBugReport":
                String prio = (String) call.argument("priority");
                BugBattle.SEVERITY severity = BugBattle.SEVERITY.LOW;
                if(prio == "MEDIUM"){
                    severity = BugBattle.SEVERITY.MIDDLE;
                }
                if(prio == "HIGH"){
                    severity = BugBattle.SEVERITY.HIGH;
                }
                BugBattle.getInstance().sendSilentBugReport((String) call.argument("email"),(String) call.argument("description"),severity);
                break;
            case "setCustomData":
                BugBattle.getInstance().setCustomData((String) call.argument("key"),(String) call.argument("value"));
                break;
            case "removeCustomData":
                BugBattle.getInstance().removeCustomDataForKey((String) call.argument("key"));
                break;
            case "clearCustomData":
                BugBattle.getInstance().clearCustomData();
                break;
            case "enablePoweredByBugbattle":
                BugBattle.getInstance().enablePoweredByBugbattle((boolean) call.argument("enabled"));
                break;
            case "setLogoUrl":
                BugBattle.getInstance().setLogoUrl((String) call.argument("logoUrl"));
                break;
            case "setColor":
                BugBattle.getInstance().setNavigationTint((String) call.argument("color"));
                break;

            default:
                result.notImplemented();
        }
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        channel.setMethodCallHandler(null);
    }
}

