#import "BugbattleSdkPlugin.h"

@interface BugbattleSdkPlugin ()

@property (retain, nonatomic) FlutterMethodChannel *methodChannel;

@end

@implementation BugbattleSdkPlugin
+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
  FlutterMethodChannel* channel = [FlutterMethodChannel
      methodChannelWithName:@"bugbattle_sdk"
            binaryMessenger:[registrar messenger]];
  BugbattleSdkPlugin* instance = [[BugbattleSdkPlugin alloc] init];
  instance.methodChannel = channel;
  [registrar addMethodCallDelegate:instance channel:channel];
}

- (UIColor *)colorFromHexCode:(NSString *)hexString {
    NSString *cleanString = [hexString stringByReplacingOccurrencesOfString:@"#" withString:@""];
    if([cleanString length] == 3) {
        cleanString = [NSString stringWithFormat:@"%@%@%@%@%@%@",
                        [cleanString substringWithRange:NSMakeRange(0, 1)],[cleanString substringWithRange:NSMakeRange(0, 1)],
                        [cleanString substringWithRange:NSMakeRange(1, 1)],[cleanString substringWithRange:NSMakeRange(1, 1)],
                        [cleanString substringWithRange:NSMakeRange(2, 1)],[cleanString substringWithRange:NSMakeRange(2, 1)]];
    }
    if([cleanString length] == 6) {
        cleanString = [cleanString stringByAppendingString:@"ff"];
    }
    
    unsigned int baseValue;
    [[NSScanner scannerWithString:cleanString] scanHexInt:&baseValue];
    
    float red = ((baseValue >> 24) & 0xFF)/255.0f;
    float green = ((baseValue >> 16) & 0xFF)/255.0f;
    float blue = ((baseValue >> 8) & 0xFF)/255.0f;
    float alpha = ((baseValue >> 0) & 0xFF)/255.0f;
    
    return [UIColor colorWithRed:red green:green blue:blue alpha:alpha];
}

- (void)initSDK {
    BugBattle.sharedInstance.delegate = self;
    [BugBattle setApplicationType: FLUTTER];
}

- (void)customActionCalled:(NSString *)customAction {
  if (self.methodChannel != nil) {
    [self.methodChannel invokeMethod: @"customActionCalled" arguments: @{
      @"name": customAction
    }];
  }
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
   if ([@"initWithToken" isEqualToString:call.method]) {
     if ([call.arguments[@"activationMethod"] isEqualToString: @"SHAKE"]) {
        [BugBattle initWithToken: call.arguments[@"apiKey"] andActivationMethod: SHAKE];
     } 
     else if ([call.arguments[@"activationMethod"] isEqualToString: @"THREE_FINGER_DOUBLE_TAB"]) {
       [BugBattle initWithToken: call.arguments[@"apiKey"] andActivationMethod: THREE_FINGER_DOUBLE_TAB];
     } 
     else if ([call.arguments[@"activationMethod"] isEqualToString: @"SCREENSHOT"]) {
       [BugBattle initWithToken: call.arguments[@"apiKey"] andActivationMethod: SCREENSHOT];
     }
     else {
        [BugBattle initWithToken: call.arguments[@"apiKey"] andActivationMethod: NONE];
     }
     [self initSDK];
  } 
  else if([@"initializeMany" isEqualToString:call.method]) {
    NSMutableArray *activationMethods = [NSMutableArray array];
    NSArray *activationMethodsAsStrings = call.arguments[@"activationMethods"];

    for(int i = 0; i < [activationMethodsAsStrings count]; i++) {
      if ([[activationMethodsAsStrings objectAtIndex:i] isEqualToString: @"SHAKE"]) {
          [activationMethods addObject: @(SHAKE)];
      } 
      else if ([[activationMethodsAsStrings objectAtIndex:i] isEqualToString: @"THREE_FINGER_DOUBLE_TAB"]) {
        [activationMethods addObject: @(THREE_FINGER_DOUBLE_TAB)];
      } 
      else if ([[activationMethodsAsStrings objectAtIndex:i] isEqualToString: @"SCREENSHOT"]) {
        [activationMethods addObject: @(SCREENSHOT)];
      }
      else {
          [activationMethods addObject: @(NONE)];
      }
    }

    [BugBattle initWithToken: call.arguments[@"apiKey"] andActivationMethods: activationMethods];
    [self initSDK];
  }
  else if([@"autoConfigure" isEqualToString:call.method]) {
    [BugBattle autoConfigureWithToken: call.arguments[@"apiKey"]];
    [self initSDK];
  } 
  else if([@"startBugReporting" isEqualToString:call.method]) {
    [BugBattle startBugReporting];
  }
  else if([@"enableReplays" isEqualToString:call.method]) {
    [BugBattle enableReplays: true];
  }
  else if([@"setCustomerEmail" isEqualToString:call.method]) {
    [BugBattle setCustomerEmail: call.arguments[@"email"]];
  }
  else if([@"sendSilentBugReport" isEqualToString:call.method]) {
    NSString *priority = call.arguments[@"priority"];
    BugBattleBugPriority prio = MEDIUM;
    if ([priority isEqualToString: @"LOW"]) {
        prio = LOW;
    }
    if ([priority isEqualToString: @"HIGH"]) {
        prio = HIGH;
    }
    [BugBattle sendSilentBugReportWith: call.arguments[@"email"] andDescription: call.arguments[@"description"] andPriority: prio];
  }
  else if([@"setLogoUrl" isEqualToString:call.method]) {
    [BugBattle setLogoUrl: call.arguments[@"logoUrl"]];
  }
  else if([@"enablePoweredByBugbattle" isEqualToString:call.method]) {
    [BugBattle enablePoweredByBugbattle: call.arguments[@"enabled"]];
  } else if([@"attachCustomData" isEqualToString:call.method]) {
    [BugBattle attachCustomData: call.arguments[@"customData"]];
  }
  else if([@"removeCustomDataForKey" isEqualToString:call.method]) {
    [BugBattle setCustomData: call.arguments[@"value"] forKey: call.arguments[@"key"]];
  }
  else if([@"removeCustomDataForKey" isEqualToString:call.method]) {
    [BugBattle removeCustomDataForKey: call.arguments[@"key"]];
  }
  else if([@"clearCustomData" isEqualToString:call.method]) {
    [BugBattle clearCustomData];
  }
  else if([@"enablePrivacyPolicy" isEqualToString:call.method]) {
    [BugBattle enablePrivacyPolicy: call.arguments[@"enable"]];
  }
  else if([@"setPrivacyPolicyUrl" isEqualToString:call.method]) {
    [BugBattle setPrivacyPolicyUrl: call.arguments[@"url"]];
  }
  else if([@"setColor" isEqualToString:call.method]) {
    [BugBattle setColor: [self colorFromHexCode: call.arguments[@"color"]]];
  }
  else if([@"setApiUrl" isEqualToString:call.method]) {
    [BugBattle setApiUrl: call.arguments[@"url"]];
  }
  else if([@"setLanguage" isEqualToString:call.method]) {
    [BugBattle setLanguage: call.arguments[@"language"]];
  }
  else {
    result(FlutterMethodNotImplemented);
  }
}

@end
